"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StripeV2 = void 0;
const Setting_entity_1 = require("./../modules/common/models/Setting.entity");
const Payment_exception_1 = require("../middleware/exception/Payment.exception");
const { Op } = require('sequelize');
class StripeV2 {
    constructor() {
        this.stripe = null;
        this.minimumAmount = 0.50;
    }
    async loadKeys() {
        let result = await Setting_entity_1.Setting.findAll({
            attributes: ['key', 'value'],
            where: {
                key: {
                    [Op.in]: ['STRIPE_SECRET_KEY', 'STRIPE_CONNECT_APPLICATION_KEY', 'STRIPE_WEBHOOK_SECRET', 'STRIPE_WEBHOOK_CONNECTED_SECRET']
                }
            }
        }).then((result) => {
            var format = {};
            for (let data of result) {
                format[data.key] = data.value;
            }
            return format;
        });
        return result;
    }
    async connect() {
        if (this.stripe === null) {
            this.stripe = require('stripe')(process.env.STRIPE_SECRET_KEY);
        }
    }
    async getStripeConnectUrl() {
        let STRIPE_APPLICATION_KEY = process.env.STRIPE_CONNECT_APPLICATION_KEY;
        return process.env.STRIPE_CONNECT_AUTHORIZE_URI + "?response_type=code&client_id=" + STRIPE_APPLICATION_KEY + "&scope=read_write";
    }
    async getAuthStripeToken(code) {
        let keys = await this.loadKeys();
        let request = require('request');
        return await new Promise(function (resolve, reject) {
            request.post(process.env.STRIPE_CONNECT_TOKEN_URI, {
                form: {
                    grant_type: 'authorization_code',
                    client_id: process.env.STRIPE_CONNECT_APPLICATION_KEY,
                    client_secret: process.env.STRIPE_SECRET_KEY,
                    code: code
                },
                json: true
            }, (err, response, body) => {
                resolve(body);
            });
        });
    }
    async createPaymentIntend(amount, currency, entity, entity_id) {
        await this.connect();
        if (amount < this.minimumAmount) {
            throw new Payment_exception_1.PaymentException("stripe:minimum:amount");
        }
        const intent = await this.stripe.paymentIntents.create({
            amount: amount,
            currency: currency,
            setup_future_usage: 'off_session',
            metadata: {
                entity_id: entity_id,
                entity: entity
            }
        });
        return intent;
    }
    async createPaymentIntendWithCustomer(amount, currency, entity, entity_id, customerId, paymentMethodId) {
        await this.connect();
        if (amount < this.minimumAmount) {
            throw new Payment_exception_1.PaymentException("stripe:minimum:amount");
        }
        const intent = await this.stripe.paymentIntents.create({
            amount: amount,
            currency: currency,
            payment_method_types: ['card'],
            customer: customerId,
            payment_method: paymentMethodId,
            off_session: true,
            confirm: true,
            metadata: {
                entity_id: entity_id,
                entity: entity
            }
        });
        return intent;
    }
    async createPaymentIntendWIthConnectedAccount(amount, currency, entity, entity_id, accountId) {
        await this.connect();
        if (amount < this.minimumAmount) {
            throw new Payment_exception_1.PaymentException("stripe:minimum:amount");
        }
        const intent = await this.stripe.paymentIntents.create({
            amount: amount,
            currency: currency,
            payment_method_types: ['card'],
            metadata: {
                entity_id: entity_id,
                entity: entity
            }
        }, {
            stripe_account: accountId,
        });
        return intent;
    }
    async processWebHook(signature, body, connected = false) {
        let keys = await this.loadKeys();
        await this.connect();
        let secret = keys['STRIPE_WEBHOOK_SECRET'];
        if (connected) {
            secret = keys['STRIPE_WEBHOOK_CONNECTED_SECRET'];
        }
        return await this.stripe.webhooks.constructEvent(body, signature, Buffer.from(secret));
    }
    async addOrUpdateCustomer(channel, intent) {
        await this.connect();
        if (channel.stripeCustomerId) {
            await this.stripe.paymentMethods.attach(intent.payment_method, {
                customer: channel.stripeCustomerId,
            });
        }
        else {
            const customer = await this.stripe.customers.create({
                payment_method: intent.payment_method,
                email: channel.companyMail,
                description: 'Customer for ' + channel.companyName,
                name: channel.companyName,
                metadata: {
                    channel_id: channel.channelId,
                    channel_name: channel.companyName
                }
            });
            channel.stripeCustomerId = customer.id;
        }
        return channel.stripeCustomerId;
    }
    async createchargesForSource(amount, currency, sourceId) {
        this.stripe = require('stripe')(process.env.STRIPE_SECRET_KEY);
        const charge = await this.stripe.charges.create({
            amount: amount,
            currency: currency,
            source: sourceId,
        });
        return charge;
    }
}
exports.StripeV2 = StripeV2;
//# sourceMappingURL=Stripe.js.map