'use strict';

import { Injectable, Inject } from '@nestjs/common';
import { Cart } from '../models/Cart.entity';
import { ProductQuote } from './../models/ProductQuote.entity';
import { ProcessException } from './../../../middleware/exception/Process.exception';
import { Product } from './../../product/models/Product.entity';
import { ProductPriceDetail } from './../../product/models/ProductPriceDetail.entity';
import { ProductGroup } from './../../product/models/ProductGroup.entity';
import { ProductCategory } from './../../product/models/ProductCategory.entity';
import { Brand } from './../../product/models/Brand.entity';
import { AuthException } from './../../../middleware/exception/Auth.exception';
import { Media } from '../../media/models/index';
import { ProductImage } from '../../product/models/ProductImage.entity';
import { BaseService } from 'src/core/service/base.service';
import { appconstant } from 'src/config/app.constant';
import { Channel } from 'src/modules/channel/models/Channel.entity';

@Injectable()
export class CartService extends BaseService {
	constructor(
		@Inject('CartRepository') private readonly repository: typeof Cart,
		@Inject('ProductQuoteRepository') private readonly quoteRepository: typeof ProductQuote,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('ProductRepository') private readonly productRepository: typeof Product,
	) {
		super();
	}

	async clearCart(userId, cartId?) {
		let where = {
			userid: userId,
			deletedat: null
		};

		if (cartId) {
			where = Object.assign(where, { 'cartid': cartId })
		}

		let result = await this.repository.findAll({
			where: where
		});

		if (!result.length) {
			throw new ProcessException('validate:field:notFound');
		}

		try {
			await this.repository.update({
				deletedat: this.sequelizeInstance.fn('NOW')
			}, { where: where });
		} catch (e) {
			throw new ProcessException('validate:sql:internalError');
		}
	}

	async _validateProduct(productId) {
		let product = await this.productRepository.findByPk(productId, {
			include: [
				{
					model: ProductPriceDetail,
					as: 'productPriceDetail',
					required: true
				}
			],
			// where: {
			// 	deletedat: null,
			// 	isactive: '1'
			// }
		});

		if (!product) {
			throw new ProcessException('validate:field:notFound');
		}

		return product;
	}

	async _getCart(sellerChannelId, buyerChannelId, userId, currencyId) {
		let cart = await this.repository.findOrCreate({
			where: {
				sellerchannelid: sellerChannelId,
				buyerchannelid: buyerChannelId,
				userid: userId,
				deletedat: null
			},
			defaults: {
				currencyid: currencyId,
				subtotal: 0,
				grandtotal: 0,
				taxcost: 0,
				discount: 0,
				createdat: this.sequelizeInstance.fn('NOW')
			}
		}).spread((data: Cart, created) => {
			return data;
		});

		if (!cart) {
			throw new ProcessException('validate:sql:internalError');
		}

		return cart;
	}

	async getAllMyCart(userId) {

		return await this.repository.findAll({
			attributes: ['cartid', 'sellerchannelid', 'currencyid', 'subtotal', 'taxcost', 'discount', 'grandtotal'],
			where: {
				deletedat: null,
				userid: userId
			},
			order: [['createdat', 'DESC']],
			include: [
				{
					model: ProductQuote,
					as: 'items',
					required: true,
					attributes: ['productid', 'quantity', 'unitprice'],
					where: {
						deletedat: null
					},
					include: [{
						model: Product,
						as: 'product',
						required: true,
						include: [
							{
								model: Brand,
								attributes: ['brandName'],							
							},
							{
							model: ProductImage.scope('productImage'),
							as: "defaultImage",
							where: { deletedat: null },
							include: [
								{
									model: Media,
									where: { deletedAt: null },
									required: false
								}
							],
							required: false
						}]
					}]
				},
				{
					model: Channel,
					attributes: ['companyName']
				}
			]
		}).then((result) => {		
			return result;//{ 'cart': formatedResult, 'inactiveProduct': inactiveProduct };
		});
	}

	async addToCart(productId, user, qty, action) {
		let product = await this._validateProduct(productId);
		let cart = await this._getCart(+product.channelId, +user.channelId, user.userId, product.productPriceDetail.businesscurrencyid);

		// don't show this product to free plan users
		if (product.isshowtofreesub == '1' && user.planId == appconstant.FREE_PLANID) {
			throw new AuthException('Not allowed');
		}

		if (await this.validateRejectedChannels(+user.channelId, +cart.sellerchannelid)) {
			throw new AuthException('Not allowed');
		}

		// new products qty validation
		var limit = product.productPriceDetail.samplethreshold;
		if (product.productPriceDetail.isunlimitsample == '1') {
			limit = appconstant.MAX_ALLOWED_QTY_FOR_UNLIMITED;
		}

		if (limit < qty) {
			throw new ProcessException('This product reached the limit');
		}

		await this.quoteRepository.findOrCreate({
			// attributes: ['quantity', 'unitprice', 'productquoteid'],
			where: {
				cartid: cart.cartid,
				productid: product.productId,
				deletedat: null
			},
			defaults: {
				quantity: qty,
				unitprice: (product.productPriceDetail.issamplefree == '1') ? 0 : product.productPriceDetail.sampleprice,
				createdat: this.sequelizeInstance.fn('NOW')
			}
		}).spread(async (data: ProductQuote, created) => {
			if (!created) {
				let quantityLimit = +data.quantity + qty;
				if (action == 'replace') {
					quantityLimit = +qty;
				}

				var limit = product.productPriceDetail.samplethreshold;
				if (product.productPriceDetail.isunlimitsample == '1') {
					limit = appconstant.MAX_ALLOWED_QTY_FOR_UNLIMITED;
				}

				if (limit < quantityLimit) {
					throw new ProcessException('This product reached the limit');
				}

				// update quantity
				data.quantity = +quantityLimit;
				data.updatedat = this.sequelizeInstance.fn('NOW');
				data.unitprice = (product.productPriceDetail.issamplefree == '1') ? 0 : +product.productPriceDetail.sampleprice;
				await data.save();
			}

			return data;
		});

		// Build a custom query to recalculate cart total
		let subtotal = await this.sequelizeInstance.query('select sum(quantity*unitprice) as subtotal from "tblproductquote" where cartid = :cartId and deletedat is null', {
			raw: true,
			type: this.sequelizeInstance.QueryTypes.SELECT,
			replacements: {
				cartId: cart.cartid
			}
		}).then((result) => {
			return +result[0].subtotal;
		});

		// update cart total
		cart.subtotal = subtotal;
		cart.updatedat = this.sequelizeInstance.fn('NOW');
		cart.grandtotal = (+cart.subtotal + +cart.taxcost) - (+cart.discount); // TODO: Flat discount only processed. discount percentage not yet implemented
		await cart.save();
	}

	async removeItemFromCart(productId, user) {
		let product = await this._validateProduct(productId);
		let cart = await this._getCart(+product.channelId, +user.channelId, user.userId, product.productPriceDetail.businesscurrencyid);

		let quote = await this.quoteRepository.findOne({
			attributes: ['productquoteid'],
			where: {
				cartid: cart.cartid,
				productid: product.productId
			}
		});

		if (!quote) {
			throw new ProcessException('This product not in the cart');
		}

		await quote.destroy();

		// Build a custom query to recalculate cart total
		let subtotal = await this.sequelizeInstance.query('select sum(quantity*unitprice) as subtotal from "tblproductquote" where cartid = :cartId and deletedat is null', {
			raw: true,
			type: this.sequelizeInstance.QueryTypes.SELECT,
			replacements: {
				cartId: cart.cartid
			}
		}).then((result) => {
			return +result[0].subtotal;
		});

		// update cart total
		cart.subtotal = subtotal;
		cart.updatedat = this.sequelizeInstance.fn('NOW');
		cart.grandtotal = (+cart.subtotal + +cart.taxcost) - (+cart.discount); // TODO: Flat discount only processed. discount percentage not yet implemented
		await cart.save();

		return;
	}


	async returnCount(userId, cartId?) {
		let query = 'select q.cartid, count(q.productid) as product_count, sum(q.quantity) as product_quantity from tblproductquote q inner join tblcart c on c.cartid = q.cartid where q.deletedat IS NULL and c.deletedat IS NULL and c.userid = :userId ';
		if (cartId) {
			query += 'and q.cartid = :cartId ';
		}

		query += 'group by q.cartid';

		let counts = await this.quoteRepository.sequelize.query(query, {
			type: this.sequelizeInstance.QueryTypes.SELECT,
			raw: true,
			replacements: {
				userId: userId,
				cartId: cartId
			}
		}).then((data) => {
			return data;
		});

		console.log("cartcount:",counts);
		return counts;
	}
}
