import { Table, Column, Model, DataType, BelongsTo, ForeignKey, AllowNull, Scopes } from 'sequelize-typescript';
import { ApiProperty } from '@nestjs/swagger';
import { PaymentType } from './PaymentType.entity';
import { Channel } from './Channel.entity';
import { User } from './../../user/models/User.entity';
import { Sequelize } from 'sequelize-typescript';
import { Currency } from './Currency.entity';
import { UUIDV4 } from 'sequelize/types';

const channelValidation = {
	isAvailable: async (value: string, next: Function): Promise<any> => {
		const isExist = await Channel.findOne({ where: { channelId: value } });
		if (!isExist) {
			next('channel:notFound');
		}
		next();
	},
};

@Table({
	timestamps: false,
	tableName: "tblPaymentTransaction"
})

@Scopes(() => ({
	channelDetail: {
		attributes: ['paymentTransactionId', 'paymentTransactionAt', 'amount', 'transactionNumber', 'cardType', 'cardLastFourDigit', 'status', 'message']
	},
}))

export class PaymentTransaction extends Model<PaymentTransaction> {
	static PAYMENT_TYPE_SUBSCRIPTION = 1;
	static PAYMENT_TYPE_TRADESHOW = 2;
	static PAYMENT_TYPE_SAMPLES = 3;
	static PAYMENT_TYPE_SALESREPPAY = 4;

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		// defaultValue:UUIDV4,
		autoIncrement: true,
		unique: true,
		primaryKey: true,
	})
	paymentTransactionId: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await PaymentType.findOne({ where: { paymentTypeId: value } });
				if (!isExist) {
					next('channel:subscription:paymentType:notFound');
				}
				next();
			},
		}
	})
	@ForeignKey(() => PaymentType)
	paymentTypeId: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false
	})
	paymentMethodId: number;

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: channelValidation
	})
	payerChannelId: number

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: channelValidation
	})
	payeeChannelId: number

	@ApiProperty()
	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	amount: number

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	transactionNumber: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	cardType: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	cardLastFourDigit: string

	@Column
	status: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	message: string

	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await User.findOne({ where: { userId: value } });
				if (!isExist) {
					next('account:resetPassword:userNotFound');
				}
				next();
			},
		}
	})
	updatedBy: number

	@Column({
		type: DataType.DATE,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		defaultValue: Sequelize.literal('CURRENT_TIMESTAMP')
	})
	paymentTransactionAt: string

	@BelongsTo(() => PaymentType)
	paymentType: PaymentType;


	@Column({
		type: DataType.INTEGER,
		allowNull: false,
		autoIncrement: false,
		unique: false,
		primaryKey: false,
		validate: {
			isAvailable: async (value: string, next: Function): Promise<any> => {
				const isExist = await Currency.findOne({ where: { currencyid: value } });
				if (!isExist) {
					next('currency:notFound');
				}
				next();
			},
		}
	})
	@ForeignKey(() => Currency)
	currencyid: number

	@BelongsTo(() => Currency)
	currency: Currency;

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	invoiceno: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	entity: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	entityId: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	couponCode: string

	@AllowNull(true)
	@Column(DataType.CHAR(255))
	paymentIntent: string
	
}
