import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, BelongsTo, ForeignKey, Scopes, HasMany, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { SubscriptionPeriod } from './SubscriptionPeriod.entity';
import { SubscriptionPlan } from './SubscriptionPlan.entity';
import { ChannelType } from './ChannelType.entity';
import { SubscriptionDetail } from './SubscriptionDetail.entity';
import { ChannelSubscription } from './ChannelSubscription.entity';
import { SubscriptionExtraFeature } from './SubscriptionExtraFeature.entity';
var Sequelize = require('sequelize');
import { ValidationException } from './../../../middleware/exception/Validation.exception';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblSubscription' } as IDefineOptions;

@Table({
	tableName: "tblSubscription"
})

@Scopes(() => ({
	all: {
		include: [SubscriptionPeriod,SubscriptionPlan]
	},
	withPeriod: {
		include: [SubscriptionPeriod]
	},
	channelDetail: {
		attributes: ['subscriptionPlanId', 'isActive', 'deletedAt']
	}
}))

export class Subscription extends Model<Subscription>{
	static SUBSCRIPTION_TYPE_NEW = 1;
	static SUBSCRIPTION_TYPE_RENEWAL = 2;
	static SUBSCRIPTION_TYPE_UPGRADE = 3;
	static SUBSCRIPTION_TYPE_DOWNGRADE = 4;

	static SUBSCRIPTION_PERIOD_MONTH = 1;
	static SUBSCRIPTION_PERIOD_YEAR = 2;

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	subscriptionId: number

	@ForeignKey(() => ChannelType)
	@Column(DataType.BIGINT)
	channelTypeId: number

	@BelongsTo(() => ChannelType)
	channelType: ChannelType;

	@ForeignKey(() => SubscriptionPeriod)
	@Column(DataType.BIGINT)
	subscriptionPeriodId: number

	@BelongsTo(() => SubscriptionPeriod)
	subscriptionPeriod: SubscriptionPeriod;

	@ForeignKey(() => SubscriptionPlan)
	@Column(DataType.BIGINT)
	subscriptionPlanId: number

	@BelongsTo(() => SubscriptionPlan)
	subscriptionPlan: SubscriptionPlan;

	@AllowNull(false)
	@Column(DataType.DECIMAL(15, 2))
	amount: number

	@HasMany(() => SubscriptionDetail, 'subscriptionId')
	subscriptionDetail: SubscriptionDetail[];

	@HasMany(() => ChannelSubscription, 'subscriptionId')
	channelSubscription: ChannelSubscription[];

	@HasMany(() => SubscriptionExtraFeature, 'subscriptionId')
	subscriptionExtraFeature: SubscriptionExtraFeature[];

	get finalAmount(): number {
		let amount: number = +this.amount
		if (this.offerPerc > 0 && amount > 0) {
			return +(amount - (amount * (this.offerPerc / 100))).toFixed(2);
		}

		return +(amount).toFixed(2);
	}

	@Column(DataType.CHAR(255))
	offerDesc: string

	@Column(DataType.DECIMAL(5, 2))
	offerPerc: number

	@Column(DataType.BOOLEAN)
	isActive: number

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column(DataType.BIGINT)
	updatedBy: number

	@BeforeCreate
	@BeforeUpdate
	public static async dateUpdate(instance: Subscription) {
		instance.updatedAt = Sequelize.fn('NOW');

		let options = {
			where: {
				channelTypeId: +instance.channelTypeId,
				subscriptionPeriodId: +instance.subscriptionPeriodId,
				subscriptionPlanId: +instance.subscriptionPlanId
			}
		};

		if (instance.subscriptionId) {
			options['where']['subscriptionId'] = {
				$not: instance.subscriptionId
			}
		}

		if (await Subscription.count(options)) {
			throw new ValidationException([
				{
					value: +instance.channelTypeId,
					property: 'channelTypeId',
					children: [],
					constraints:
						{ unique: 'validate:field:shouldBeUnique' }
				},
				{
					value: +instance.subscriptionPeriodId,
					property: 'subscriptionPeriodId',
					children: [],
					constraints:
						{ unique: 'validate:field:shouldBeUnique' }
				},
				{
					value: +instance.subscriptionPlanId,
					property: 'subscriptionPlanId',
					children: [],
					constraints:
						{ unique: 'validate:field:shouldBeUnique' }
				}
			]);
		}
	}
}
