'use strict';
import { Table, Column, PrimaryKey, Model, AutoIncrement, DataType, AllowNull, ForeignKey, Scopes, BelongsTo, BeforeCreate, BeforeUpdate } from 'sequelize-typescript';
import { IDefineOptions } from 'sequelize-typescript/node_modules/glob';
import { ProductGroup } from './ProductGroup.entity';
import { ValidationException } from './../../../middleware/exception/Validation.exception';

const tableOptions: IDefineOptions = { timestamp: true, tableName: 'tblProductCategories' } as IDefineOptions;
@Table(tableOptions)
@Scopes(() => ({
	productCategoryInfo: {
		attributes: [['productCategorieId', 'id'], ['productCategorie', 'name']]
	},
	channelDetail: {
		attributes: ['productCategorieId', 'productGroupId', 'productCategorie', 'deletedAt', 'isActive']
	},
	contactProductInfom: {
		attributes: ['productCategorieId', 'productGroupId', 'productCategorie', 'isActive']
	},
	tradeinfo: {
		attributes: {
			include: [['productCategorieId', 'productCategorieId'], 'productGroupId', 'productCategorie', 'isActive'],
			exclude: ['deletedAt', 'createdAt', 'updatedAt']
		}
	},
	channelDetailList: {
		attributes: ['productCategorie']
	},
	excludeDeleted: {
		where: {
			deletedAt: null
		},
		attributes: ['productCategorieId', 'productGroupId', 'productCategorie', 'isActive']
	}
}))

export class ProductCategory extends Model<ProductCategory>{

	@PrimaryKey
	@AutoIncrement
	@AllowNull(false)
	@Column(DataType.BIGINT)
	productCategorieId: number

	@ForeignKey(() => ProductGroup)
	@Column({
		type: DataType.BIGINT,
		allowNull: false,
		unique: { name: 'productCategories', msg: 'validate:field:shouldBeUnique' }
	})
	productGroupId: number;

	@BelongsTo(() => ProductGroup)
	productGroup: ProductGroup;

	@Column({
		type: DataType.CHAR(255),
		allowNull: false,
		unique: { name: 'productCategories', msg: 'validate:field:shouldBeUnique' }
	})
	productCategorie: string;

	@Column
	isActive: string;

	@Column(DataType.DATE)
	deletedAt: Date;

	@Column(DataType.DATE)
	createdAt: Date;

	@Column(DataType.DATE)
	updatedAt: Date;

	@Column
	businessType: string;

	@BeforeCreate
	@BeforeUpdate
	static async makeUniqueValidation(instance: ProductCategory) {
		let options = {
			where: {
				productCategorie: {
					$ilike: instance.productCategorie.trim()
				},
				productGroupId: instance.productGroupId,
				deletedAt: null
			}
		};

		if (instance.productCategorieId) {
			options['where']['productCategorieId'] = {
				$not: instance.productCategorieId
			}
		}

		if (await ProductCategory.count(options)) {
			throw new ValidationException([{
				value: instance.productCategorie.trim(),
				property: 'productCategorie',
				children: [],
				constraints:
					{ unique: 'validate:field:shouldBeUnique' }
			}]);
		}
	}
}
