'use strict';

import { Injectable, Inject } from '@nestjs/common';
import { Wishlist } from '../models/Wishlist.entity';
import { Channelgroup } from '../models/Channelgroup.entity';

import { ProductEvaluation } from './../../product/models/ProductEvaluation.entity';
import { ProcessException } from './../../../middleware/exception/Process.exception';
import { AuthException } from './../../../middleware/exception/Auth.exception';
import { Product } from './../../product/models/Product.entity';
import { ChannelJCT } from './../../channel/models/ChannelJCT.entity';
import { Sequelize } from 'sequelize-typescript';
import { isNull } from 'lodash';
import {Op, QueryTypes } from "sequelize";
import { BaseService } from 'src/core/service/base.service';
import { appconstant } from 'src/config/app.constant';
import { ProductImage } from 'src/modules/product/models/ProductImage.entity';
import { Media } from 'src/modules/media/models';
import { ProductAward } from 'src/modules/product/models/ProductAward.entity';
import { Brand } from 'src/modules/product/models/Brand.entity';

@Injectable()
export class WishlistService extends BaseService {
	constructor(
		@Inject('WishlistRepository') private readonly repository: typeof Wishlist,				
		@Inject('ChannelGroupRepository') private readonly grouprepository: typeof Channelgroup,	
		@Inject('ProductEvaluationRepository') private readonly evaluation: typeof ProductEvaluation,
		@Inject('SequelizeInstance') private readonly sequelizeInstance,
		@Inject('ProductRepository') private readonly productRepository: typeof Product,
		@Inject('ProductImageRepository') private readonly productImageRepository: typeof ProductImage,
		@Inject('MediaRepository') private readonly mediaRepository: typeof Media,
		@Inject('ProductAwardRepository') private readonly productAwardRepository: typeof ProductAward,
		@Inject('BrandRepository') private readonly brandRepository: typeof Brand,
	) {
		super();
	}

	async create(data, user): Promise<any> {

		let product = await Product.findOne({
			where: {
				productId: data.productid,
				deletedat: null
			}
		});

		if (!product) {
			throw new AuthException('Not allowed');
		}

		// don't show this product to free plan users
		if (product.isshowtofreesub == '1' && user.planId == appconstant.FREE_PLANID) {
			throw new AuthException('Not allowed');
		}


		if (await this.validateRejectedChannels(+data.channelid, +product.channelId)) {
			throw new AuthException('Not allowed');
		}

		try {
			return await this.repository.create<Wishlist>(this._buildObject(new Object(), data)).then(async (result) => {
				return result;
			});
		} catch (e) {
			let message = 'validate:sql:internalError';
			if (e.errors && (e.errors[0].message == 'productid must be unique' || e.errors[0].message == 'type must be unique')) {
				message = 'this product already in ' + data.type;
			} else {
				console.log(e.errors);
			}

			throw new ProcessException(message);
		}
	}
	//create new channel group --@Nagaraj
	async createChannelGroup(data,user): Promise<any> {
		try {
			return await this.grouprepository.create<Channelgroup>(this._groupObject(new Object(), data)).then(async (result) => {
				return result;
			});
		} catch (e) {
			let message = 'validate:sql:internalError';
			throw new ProcessException(message);
		}
	}
	//add favorite channel to group --@Nagaraj
	async createFavoriteChannel(data,user,list_type): Promise<any> {
		try {
			return await this.repository.create<Wishlist>(this._favoriteObject(new Object(), data)).then(async (result) => {
			 	return result;
			 });
		} catch (e) {
			console.log(e.message);
			let message = 'validate:sql:internalError';
			throw new ProcessException(e.message);
		}
	}
	//remove favorite channel from group --@Nagaraj
	async removeFavoriteChannel(data,user,list_type): Promise<any> {

		await this.repository.update({ deletedat: new Date() }, {
			where: {
				channelid: data.channelid,
				userid: data.userid,				
				channelgroupid: data.channelgroupid,
				deletedat: null
			}//, returning: true
		});
	}

	//remove favorite group from group --@Nagaraj
	async removeFavoriteGroup(id:number): Promise<any> {
		await this.grouprepository.update({ deletedAt: new Date() }, {
			where: {							
				channelGroupId: id,
				deletedAt: null
			}//, returning: true
		});
	}

	//get favorite group list --@Nagaraj
	async getGroupList(user) {	
			
			let result=[];
		let gtotal = 0;
			let group = await this.grouprepository.findAll<Channelgroup>({		
				
				where: {			
					channelId:user.channelId,		
					deletedAt: null					
				},
				attributes: ['channelGroupId', 'channelName'],
				order: [['channelGroupId', 'DESC']]
			});	

			
		for (var i = 0, len = group.length; i < len; i++) {
				
				let total=0;			
				total=await this.repository.count({
				where: {						
						channelgroupid: group[i].channelGroupId,
						userid: user.userId,
					deletedat: null
				}					
				});

			if (total > 0) {
				gtotal = gtotal + total;
			result.push({id:group[i].channelGroupId,name:group[i].channelName,count:total});
			}
		}
		
		let response = {
			result: result,
			gtotal: gtotal
		}
		return response;
	}

	
	public async findAll(userId, type, channelId, rejectedProductIds?): Promise<Array<Wishlist>> {
		let result;
		if (type == 'EVALVATED') {
			result = await this.evaluation.findAll({
				where: {
					channelid: channelId,
					productid: {
						[Op.not]: rejectedProductIds
					}
				},
				attributes: ['productid'],
				order: [['productevaluationid', 'DESC']]
			});
		} else {
			result = await this.repository.findAll<Wishlist>({
				where: {
					userid: userId,
					deletedat: null,
					channelgroupid:null,
					type: type,
					productid: {
						[Op.not]: rejectedProductIds
					}
				},
				attributes: ['productid'],
				order: [['productwishlistid', 'DESC']]
			});
		}

		return result;
	}

	public async move(channelId, userId, product, user) {
		let count = await this.repository.count({
			where: {
				channelid: channelId,
				productid: product.id,
				type: product.to_type
			}
		});

		if (count > 0) {
			throw new ProcessException("Already in BUSINESS");
		}

		let productData = await Product.findOne({
			where: {
				productId: product.id,
				deletedat: null
			}
		});
		if (!product) {
			throw new AuthException('Not allowed');
		}

		let evaluateQuery = 'select count(pe.productid) from "tblproductevaluation" pe where pe.channelid = :cahnnelId and pe."productid"= :productId';
		let evaluated = await this.sequelizeInstance.query(evaluateQuery, {
			type: this.sequelizeInstance.QueryTypes.SELECT,
			raw: true,
			replacements: {
				productId: productData.productId,
				cahnnelId: channelId
			}
		}).then(result => {
			return !!(+result[0].count);
		});
		if (!evaluated) {
			throw new ProcessException("Product not yet evaluated");
		}
		return await this.create({
			userid: userId,
			channelid: channelId,
			productid: productData.productId,
			type: 'BUSINESS'
		}, user).then(async (data) => {
			// change the relation tag
			await ChannelJCT.findOne({
				where: {
					[Op.or]: [
						{
							partnerChannelId: +user.channelId,
							channelId: +productData.channelId
						},
						{
							partnerChannelId: +productData.channelId,
							channelId: +user.channelId
						}
					]
				}
			}).then(async (result) => {
				if (result && result.channelJCTId) {
					if (result.channelId == +user.channelId) {
						result.leadManagementTag = 'High';
					} else if (result.partnerChannelId == +user.channelId) {
						result.partnerLeadManagementTag = 'High';
					}
					await result.save();
				}
			})
		}).then(async (result) => {

			// let requestObject = <IMailNotification>{};

			// requestObject.senderId = user.contactId;
			// requestObject.receiverId = await this.sequelizeInstance.query('select "contactId", "isDefaultContact", "workNatureId"  from "tblContact" CON where "channelId" = :partnerChannelId', {
			// 	type: Sequelize.QueryTypes.SELECT,
			// 	replacements: {
			// 		partnerChannelId: +productData.channelId
			// 	}
			// }).then((result) => {
			// 	let otherUsers = [];
			// 	if (result && result.length > 0) {
			// 		if (result.length == 1) {
			// 			return +result[0].contactId;
			// 		} else {
			// 			result.forEach((contact) => {
			// 				if (contact.isDefaultContact == '1') {
			// 					otherUsers = [contact];
			// 					return false;
			// 				} else if (contact.workNatureId != null) {
			// 					otherUsers.push(contact);
			// 				}
			// 			});

			// 			return +otherUsers[0].contactId;
			// 		}
			// 	} else {
			// 		return null;
			// 	}
			// });

			// if (isNull(requestObject.receiverId)) {
			// 	delete requestObject.receiverId;
			// }

			// requestObject.senderId = +user.contactId;
			// requestObject.productId = productData.productId;
			// this.notificationService.sendNotificationRouter(requestObject, notification.PRODUCT_READY_FOR_BUSSINESS);
		}).then(async (result) => {
			return result;
		});
	}

	public async delete(id: number, type: string, user: any): Promise<void> {
		let result = await Wishlist.destroy({
			where: {
				userid: +user.userId,
				productid: id,
				channelgroupid:null,
				deletedat: null,
				type: type
			}
		});

		if (!result) {
			throw new ProcessException('validate:field:notFound');
		}

		try {
			//result.destroy().then(async (result) => {
				// if(result)
				// return result;
			//});
		} catch (e) {
			throw new ProcessException('validate:sql:internalError');
		}

		return;
	}

	private _buildObject(entity, dto) {
		entity.userid = dto.userid;
		entity.channelid = dto.channelid;
		entity.productid = dto.productid;
		entity.createdat = this.sequelizeInstance.fn('NOW');
		entity.type = dto.type;

		return entity;
	}

	private _groupObject(entity, dto) {
		entity.userId = dto.userId;
		entity.channelId = dto.channelId;	
		entity.channelName = dto.groupName;		
		entity.createdAt = this.sequelizeInstance.fn('NOW');

		return entity;
	}

	private _favoriteObject(entity, dto) {
		entity.userid = dto.userid;
		entity.channelid = dto.channelid;	
		entity.channelgroupid = dto.channelgroupid;				
		entity.createdat = this.sequelizeInstance.fn('NOW');
		entity.type = dto.type;
		
		return entity;
	}


	async count(type, userId) {
		return await this.repository.count({
			where: {
				type: type,
				userid: userId,
				deletedat: null
			},
			include: [{
				model: Product,
				where: {
					deletedat: null,
					isactive: '1'
				}
			}]
		});
	}

	async channelcount(user) {

		// let group = await this.grouprepository.findAll<Channelgroup>({

		// 	where: {
		// 		channelId: user.channelId,
		// 		deletedAt: null
		// 	},
		// 	attributes: ['channelGroupId', 'channelName'],
		// 	order: [['channelGroupId', 'DESC']]
		// });

		var total = 0;
//		for (var i = 0, len = group.length; i < len; i++) {

		total = await this.repository.count({
				where: {
					//channelgroupid: group[i].channelGroupId,
					userid: user.userId,
					productid: null,
					deletedat: null
				}
			});

			//console.log("group:",group[i]["channelGroupId"],"Total:",total);

//		}

		return total;
	}

	async productcount(user, productId) {
		let total = 0;
		total = await this.repository.count({
			where: {
				channelid: user.channelId,
				productid: productId,
				type: 'WISHLIST',
				deletedat: null
			}
		});
		return total;
	}
	async getSuggestedProduct(ids,data, user){   
    
    console.log("ids:",ids);
	try {
		return this.productRepository.scope('productInfo').findAll<Product>({
			limit: data.limit,
			offset: data.offset,
			where :{            
				//isInovation: (viewId==20)?'1':{[Op.or]:['0',null]},
				deletedat: null,
				productId: {
					[Op.in]:ids
				}
			},
			include: [
				{
					model: this.productImageRepository,
					as: 'defaultImage',
					attributes: ['productimageid', 'imageid', 'productimagetype', 'name', 'description'],
					where: { deletedat: null },
					required: false,
					include: [{
						model: this.mediaRepository,
						attributes: ['documentUrl', 'documentPath'],
						where: { deletedAt: null },
						required: false,
					}]
				},
				{
					model: this.productAwardRepository,                
					attributes: ['awardDetail'],
					required: false,
					where: {
						deletedAt: null,                          
					},
					include: [
						{
							model: this.mediaRepository,
							attributes: ['documentPath'],
							required: false,
						}
					]
				},
				{
					model: this.brandRepository,                
					attributes: ['brandName'],
					required: false,
					where: {
						deletedAt: null,                          
					},
					include: [
						{
							model: this.mediaRepository,
							attributes: ['documentPath'],
							required: false,
						}
					]
				},
				{
					model: Wishlist,      
					as:'wishList',                  
					attributes: ['productid'],
					where: { 
						userid: user.userId,
						deletedat: null
						},
					required: false,
				}
			]
	
		});
	} catch (error) {
		console.log('error',error);
		throw new ProcessException(error);
	}
}
}
